@@ -226,6 +226,9 @@ func (p *nodeInterp) Caller(node ast.Node) string {
 
 func (p *nodeInterp) LoadExpr(node ast.Node) string {
 	start := node.Pos()
+	if start == token.NoPos {
+		return ""
+	}
 	pos := p.fset.Position(start)
 	f := p.files[pos.Filename]
 	n := int(node.End() - start)
@@ -365,6 +368,7 @@ type blockCtx struct {
 	pystr_     gogen.Ref
 	relBaseDir string
 
+	classDecl *ast.GenDecl   // available when isClass
 	classRecv *ast.FieldList // available when isClass
 	baseClass types.Object   // available when isClass
 
@@ -409,11 +413,11 @@ func (p *pkgCtx) newCodeError(pos token.Pos, msg string) error {
 	return &gogen.CodeError{Fset: p.nodeInterp, Pos: pos, Msg: msg}
 }
 
-func (p *pkgCtx) newCodeErrorf(pos token.Pos, format string, args ...interface{}) error {
+func (p *pkgCtx) newCodeErrorf(pos token.Pos, format string, args ...any) error {
 	return &gogen.CodeError{Fset: p.nodeInterp, Pos: pos, Msg: fmt.Sprintf(format, args...)}
 }
 
-func (p *pkgCtx) handleErrorf(pos token.Pos, format string, args ...interface{}) {
+func (p *pkgCtx) handleErrorf(pos token.Pos, format string, args ...any) {
 	p.handleErr(p.newCodeErrorf(pos, format, args...))
 }
 
@@ -460,12 +464,12 @@ func (p *pkgCtx) loadSymbol(name string) bool {
 	return false
 }
 
-func (p *pkgCtx) handleRecover(e interface{}, src ast.Node) {
+func (p *pkgCtx) handleRecover(e any, src ast.Node) {
 	err := p.recoverErr(e, src)
 	p.handleErr(err)
 }
 
-func (p *pkgCtx) recoverErr(e interface{}, src ast.Node) error {
+func (p *pkgCtx) recoverErr(e any, src ast.Node) error {
 	err, ok := e.(error)
 	if !ok {
 		if src != nil {
@@ -724,6 +728,7 @@ func preloadGopFile(p *gogen.Package, ctx *blockCtx, file string, f *ast.File, c
 	var baseTypeName string
 	var baseType types.Type
 	var spxProj string
+	var spfeats spriteFeat
 	var spxClass bool
 	var goxTestFile bool
 	var parent = ctx.pkgCtx
@@ -760,7 +765,9 @@ func preloadGopFile(p *gogen.Package, ctx *blockCtx, file string, f *ast.File, c
 				sp := proj.sprite[c.ext]
 				o := sp.obj
 				ctx.baseClass = o
-				baseTypeName, baseType, spxProj, spxClass = o.Name(), o.Type(), sp.proj, true
+				baseTypeName, baseType, spxProj = o.Name(), o.Type(), sp.proj
+				spxClass = true
+				spfeats = proj.spfeats
 			}
 		}
 	}
@@ -777,7 +784,7 @@ func preloadGopFile(p *gogen.Package, ctx *blockCtx, file string, f *ast.File, c
 		}
 		syms := parent.syms
 		pos := f.Pos()
-		specs := getFields(f)
+		ctx.classDecl = f.ClassFieldsDecl()
 		ld := getTypeLoader(parent, syms, pos, classType)
 		ld.typ = func() {
 			if debugLoad {
@@ -818,32 +825,34 @@ func preloadGopFile(p *gogen.Package, ctx *blockCtx, file string, f *ast.File, c
 					}
 				}
 				rec := ctx.recorder()
-				for _, v := range specs {
-					spec := v.(*ast.ValueSpec)
-					typ := toType(ctx, spec.Type)
-					tag := toFieldTag(spec.Tag)
-					if len(spec.Names) == 0 {
-						name := parseTypeEmbedName(spec.Type)
-						if chk.chkRedecl(ctx, name.Name, spec.Type.Pos()) {
-							continue
-						}
-						fld := types.NewField(spec.Type.Pos(), pkg, name.Name, typ, true)
-						if rec != nil {
-							rec.Def(name, fld)
-						}
-						flds = append(flds, fld)
-						tags = append(tags, tag)
-					} else {
-						for _, name := range spec.Names {
-							if chk.chkRedecl(ctx, name.Name, name.Pos()) {
+				if classDecl := ctx.classDecl; classDecl != nil {
+					for _, v := range classDecl.Specs {
+						spec := v.(*ast.ValueSpec)
+						typ := toType(ctx, spec.Type)
+						tag := toFieldTag(spec.Tag)
+						if len(spec.Names) == 0 {
+							name := parseTypeEmbedName(spec.Type)
+							if chk.chkRedecl(ctx, name.Name, spec.Type.Pos()) {
 								continue
 							}
-							fld := types.NewField(name.Pos(), pkg, name.Name, typ, false)
+							fld := types.NewField(spec.Type.Pos(), pkg, name.Name, typ, true)
 							if rec != nil {
 								rec.Def(name, fld)
 							}
 							flds = append(flds, fld)
 							tags = append(tags, tag)
+						} else {
+							for _, name := range spec.Names {
+								if chk.chkRedecl(ctx, name.Name, name.Pos()) {
+									continue
+								}
+								fld := types.NewField(name.Pos(), pkg, name.Name, typ, false)
+								if rec != nil {
+									rec.Def(name, fld)
+								}
+								flds = append(flds, fld)
+								tags = append(tags, tag)
+							}
 						}
 					}
 				}
@@ -863,10 +872,6 @@ func preloadGopFile(p *gogen.Package, ctx *blockCtx, file string, f *ast.File, c
 				X: &ast.Ident{Name: classType},
 			},
 		}}}
-		// func Classfname() string
-		if spxClass {
-			f.Decls = append(f.Decls, astFnClassfname(c))
-		}
 	}
 
 	if d := f.ShadowEntry; d != nil {
@@ -876,6 +881,25 @@ func preloadGopFile(p *gogen.Package, ctx *blockCtx, file string, f *ast.File, c
 	}
 
 	preloadFile(p, ctx, f, goFile, !conf.Outline)
+	if spfeats != 0 {
+		ld := getTypeLoader(parent, parent.syms, token.NoPos, classType)
+		if (spfeats & spriteClassfname) != 0 { // Classfname() string
+			ld.methods = append(ld.methods, func() {
+				old, _ := p.SetCurFile(goFile, true)
+				defer p.RestoreCurFile(old)
+				doInitType(ld)
+				genClassfname(ctx, c)
+			})
+		}
+		if (spfeats & spriteClassclone) != 0 { // Classclone() clonetype
+			ld.methods = append(ld.methods, func() {
+				old, _ := p.SetCurFile(goFile, true)
+				defer p.RestoreCurFile(old)
+				doInitType(ld)
+				genClassclone(ctx, proj.classclone)
+			})
+		}
+	}
 	if goxTestFile {
 		parent.inits = append(parent.inits, func() {
 			old, _ := p.SetCurFile(testingGoFile, true)
@@ -901,13 +925,10 @@ retry:
 
 func preloadFile(p *gogen.Package, ctx *blockCtx, f *ast.File, goFile string, genFnBody bool) {
 	parent := ctx.pkgCtx
+	classDecl := ctx.classDecl
 	syms := parent.syms
 	old, _ := p.SetCurFile(goFile, true)
 	defer p.RestoreCurFile(old)
-	var skipClassFields bool
-	if f.IsClass {
-		skipClassFields = true
-	}
 
 	preloadFuncDecl := func(d *ast.FuncDecl) {
 		if ctx.classRecv != nil { // in class file (.spx/.gmx)
@@ -1019,7 +1040,12 @@ func preloadFile(p *gogen.Package, ctx *blockCtx, f *ast.File, goFile string, ge
 								if debugLoad {
 									log.Println("==> Load > AliasType", name)
 								}
-								defs.AliasType(name, toType(ctx, t.Type), tName)
+								typ := defs.AliasType(name, toType(ctx, t.Type), tName)
+								if rec := ctx.recorder(); rec != nil {
+									if obj, ok := typ.(interface{ Obj() *types.TypeName }); ok {
+										rec.Def(tName, obj.Obj())
+									}
+								}
 								return
 							}
 							if debugLoad {
@@ -1069,8 +1095,7 @@ func preloadFile(p *gogen.Package, ctx *blockCtx, f *ast.File, goFile string, ge
 			case token.CONST:
 				preloadConst(d)
 			case token.VAR:
-				if skipClassFields {
-					skipClassFields = false
+				if d == classDecl { // skip class fields
 					continue
 				}
 				for _, spec := range d.Specs {
@@ -1298,7 +1323,7 @@ func loadFunc(ctx *blockCtx, recv *types.Var, name string, d *ast.FuncDecl, genB
 	var pkg = ctx.pkg
 	var sigBase *types.Signature
 	if d.Shadow {
-		if recv != nil {
+		if recv != nil && (name == "Main" || name == "MainEntry") {
 			if base := ctx.baseClass; base != nil {
 				if f := findMethod(base, name); f != nil {
 					sigBase = makeMainSig(recv, f)
