@@ -32,6 +32,7 @@ import (
 	"github.com/goplus/gop/ast"
 	"github.com/goplus/gop/printer"
 	"github.com/goplus/gop/token"
+	tpl "github.com/goplus/gop/tpl/ast"
 )
 
 /*-----------------------------------------------------------------------------
@@ -387,6 +388,8 @@ func compileExpr(ctx *blockCtx, expr ast.Expr, inFlags ...int) {
 		compileEnvExpr(ctx, v)
 	/* case *ast.MatrixLit:
 	compileMatrixLit(ctx, v) */
+	case *ast.DomainTextLit:
+		compileDomainTextLit(ctx, v)
 	default:
 		panic(ctx.newCodeErrorf(v.Pos(), "compileExpr failed: unknown - %T", v))
 	}
@@ -871,14 +874,14 @@ retry:
 
 func compileLambda(ctx *blockCtx, lambda ast.Expr, sig *types.Signature) {
 	switch expr := lambda.(type) {
-	case *ast.LambdaExpr:
-		if err := compileLambdaExpr(ctx, expr, sig); err != nil {
-			panic(err)
-		}
 	case *ast.LambdaExpr2:
 		if err := compileLambdaExpr2(ctx, expr, sig); err != nil {
 			panic(err)
 		}
+	case *ast.LambdaExpr:
+		if err := compileLambdaExpr(ctx, expr, sig); err != nil {
+			panic(err)
+		}
 	}
 }
 
@@ -1071,6 +1074,98 @@ func compileStringLitEx(ctx *blockCtx, cb *gogen.CodeBuilder, lit *ast.BasicLit)
 	}
 }
 
+const (
+	tplPkgPath = "github.com/goplus/gop/tpl"
+)
+
+// https://github.com/goplus/gop/issues/2143
+// domainTag`...` => domainTag.new(`...`)
+func compileDomainTextLit(ctx *blockCtx, v *ast.DomainTextLit) {
+	var cb = ctx.cb
+	var imp gogen.PkgRef
+	var name = v.Domain.Name
+	var path string
+	if pi, ok := ctx.findImport(name); ok {
+		imp = pi.PkgRef
+		path = pi.Path()
+		if path == "golang.org/x/net/html" {
+			// html`...` => html.Parse(strings.NewReader(`...`))
+			cb.Val(imp.Ref("Parse")).
+				Val(ctx.pkg.Import("strings").Ref("NewReader")).
+				Val(&goast.BasicLit{Kind: gotoken.STRING, Value: v.Value}, v).
+				CallWith(1, 0, v).
+				CallWith(1, 0, v)
+			return
+		}
+	} else {
+		if name == "tpl" {
+			path = tplPkgPath
+		} else {
+			path = tplPkgPath + "/encoding/" + name
+		}
+		imp = ctx.pkg.Import(path)
+		/* TODO(xsw):
+		if imp = ctx.pkg.TryImport(path); imp.Types == nil {
+			panic("compileDomainTextLit TODO: unknown domain: " + name)
+		}
+		*/
+	}
+
+	n := 1
+	if path == tplPkgPath {
+		pos := ctx.fset.Position(v.ValuePos)
+		filename := relFile(ctx.relBaseDir, pos.Filename)
+		cb.Val(imp.Ref("NewEx")).
+			Val(&goast.BasicLit{Kind: gotoken.STRING, Value: v.Value}, v).
+			Val(filename).Val(pos.Line).Val(pos.Column)
+		n += 3
+		if f, ok := v.Extra.(*tpl.File); ok {
+			decls := f.Decls
+			for _, decl := range decls {
+				if r, ok := decl.(*tpl.Rule); ok {
+					if expr, ok := r.RetProc.(*ast.LambdaExpr2); ok {
+						cb.Val(r.Name.Name)
+						sig := sigRetFunc(ctx.pkg, r.IsList())
+						compileLambdaExpr2(ctx, lambdaRetFunc(expr), sig)
+						n += 2
+					}
+				}
+			}
+		}
+	} else {
+		cb.Val(imp.Ref("New")).
+			Val(&goast.BasicLit{Kind: gotoken.STRING, Value: v.Value}, v)
+	}
+	cb.CallWith(n, 0, v)
+}
+
+func lambdaRetFunc(expr *ast.LambdaExpr2) *ast.LambdaExpr2 {
+	v := *expr
+	v.Lhs = []*ast.Ident{
+		{NamePos: expr.Pos(), Name: "self"},
+	}
+	return &v
+}
+
+func sigRetFunc(pkg *gogen.Package, isList bool) *types.Signature {
+	rets := types.NewTuple(anyParam(pkg))
+	var args *types.Tuple
+	if isList {
+		args = types.NewTuple(anySliceParam(pkg))
+	} else {
+		args = rets
+	}
+	return types.NewSignatureType(nil, nil, nil, args, rets, false)
+}
+
+func anyParam(pkg *gogen.Package) *types.Var {
+	return pkg.NewParam(token.NoPos, "", gogen.TyEmptyInterface)
+}
+
+func anySliceParam(pkg *gogen.Package) *types.Var {
+	return pkg.NewParam(token.NoPos, "", types.NewSlice(gogen.TyEmptyInterface))
+}
+
 const (
 	compositeLitVal    = 0
 	compositeLitKeyVal = 1
@@ -1262,22 +1357,25 @@ func compileCompositeLitEx(ctx *blockCtx, v *ast.CompositeLit, expected types.Ty
 			return err
 		}
 		n := len(v.Elts)
-		if isMap(underlying) {
+		switch underlying.(type) {
+		case *types.Slice:
+			ctx.cb.SliceLitEx(typ, n<<kind, kind == compositeLitKeyVal, v)
+		case *types.Array:
+			ctx.cb.ArrayLitEx(typ, n<<kind, kind == compositeLitKeyVal, v)
+		case *types.Map:
 			if kind == compositeLitVal && n > 0 {
 				return ctx.newCodeError(v.Pos(), "missing key in map literal")
 			}
-			if err := ctx.cb.MapLitEx(typ, n<<1, v); err != nil {
+			if err := compileMapLitEx(ctx, typ, n, v); err != nil {
 				return err
 			}
-		} else {
-			switch underlying.(type) {
-			case *types.Slice:
-				ctx.cb.SliceLitEx(typ, n<<kind, kind == compositeLitKeyVal, v)
-			case *types.Array:
-				ctx.cb.ArrayLitEx(typ, n<<kind, kind == compositeLitKeyVal, v)
-			default:
+		default:
+			if kind == compositeLitVal && n > 0 {
 				return ctx.newCodeErrorf(v.Pos(), "invalid composite literal type %v", typ)
 			}
+			if err := compileMapLitEx(ctx, nil, n, v); err != nil {
+				return err
+			}
 		}
 	}
 	if hasPtr {
@@ -1290,12 +1388,14 @@ func compileCompositeLitEx(ctx *blockCtx, v *ast.CompositeLit, expected types.Ty
 	return nil
 }
 
-func isMap(tu types.Type) bool {
-	if tu == nil { // map can omit type
-		return true
-	}
-	_, ok := tu.(*types.Map)
-	return ok
+func compileMapLitEx(ctx *blockCtx, typ types.Type, n int, v *ast.CompositeLit) (err error) {
+	defer func() {
+		if e := recover(); e != nil {
+			err = ctx.newCodeError(v.Pos(), "invalid map literal")
+		}
+	}()
+	err = ctx.cb.MapLitEx(typ, n<<1, v)
+	return
 }
 
 func isMapOrStruct(tu types.Type) bool {
@@ -1320,10 +1420,10 @@ func compileSliceLit(ctx *blockCtx, v *ast.SliceLit, typ types.Type, noPanic ...
 	for _, elt := range v.Elts {
 		compileExpr(ctx, elt)
 	}
-	if sliceHasTypeParam(ctx, typ) {
-		ctx.cb.SliceLitEx(nil, n, false, v)
-	} else {
+	if isSpecificSliceType(ctx, typ) {
 		ctx.cb.SliceLitEx(typ, n, false, v)
+	} else {
+		ctx.cb.SliceLitEx(nil, n, false, v)
 	}
 	return
 }
@@ -1473,8 +1573,12 @@ func compileErrWrapExpr(ctx *blockCtx, v *ast.ErrWrapExpr, inFlags int) {
 	if !useClosure && (cb.Scope().Parent() == types.Universe) {
 		panic("TODO: can't use expr? in global")
 	}
-
-	compileExpr(ctx, v.X, inFlags)
+	expr := v.X
+	switch expr.(type) {
+	case *ast.Ident, *ast.SelectorExpr:
+		expr = &ast.CallExpr{Fun: expr}
+	}
+	compileExpr(ctx, expr, inFlags)
 	x := cb.InternalStack().Pop()
 	n := 0
 	results, ok := x.Type.(*types.Tuple)
